package cn.tonyandmoney.tuon.core.config

import cn.tonyandmoney.tuon.core.cros.GlobalCrosFilter
import cn.tonyandmoney.tuon.core.cros.GlobalWebCrosFilter
import cn.tonyandmoney.tuon.core.properties.CustomConfigProperties
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer
import com.fasterxml.jackson.module.kotlin.registerKotlinModule
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.autoconfigure.condition.ConditionalOnWebApplication
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.Primary
import org.springframework.http.converter.json.Jackson2ObjectMapperBuilder
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter
import org.springframework.http.converter.xml.MappingJackson2XmlHttpMessageConverter
import org.springframework.web.server.WebFilter
import java.text.SimpleDateFormat


/**
 * 通用的核心配置
 */
@Configuration
@EnableConfigurationProperties(value = [CustomConfigProperties::class])
class CustomCoreConfiguration {

    companion object {
        const val CONVERTER_NAME = "hookMappingJackson2HttpMessageConverter"
    }


    /**
     * 跨域配置，Reactive
     */
    @Bean
    @ConditionalOnWebApplication(type = ConditionalOnWebApplication.Type.REACTIVE)
    @ConditionalOnProperty(prefix = CustomConfigProperties.CROS_PREFIX, name = ["enable"], havingValue = "true", matchIfMissing = true)
    fun fluxCrosFilter(): WebFilter {
        return GlobalCrosFilter()
    }

    /**
     * servlet跨域配置
     */
    @Bean
    @ConditionalOnWebApplication(type = ConditionalOnWebApplication.Type.SERVLET)
    @ConditionalOnProperty(prefix = CustomConfigProperties.CROS_PREFIX, name = ["enable"], havingValue = "true", matchIfMissing = true)
    fun crosFilter(): GlobalWebCrosFilter {
        return GlobalWebCrosFilter()
    }


    @Bean
    @Primary
    fun builder(): Jackson2ObjectMapperBuilder {
        return Jackson2ObjectMapperBuilder().apply {
            serializerByType(java.lang.Long::class.java, ToStringSerializer.instance)
            serializerByType(java.lang.Long.TYPE, ToStringSerializer.instance)
            // date
            dateFormat(SimpleDateFormat("yyyy-MM-dd HH:mm:ss"))
            timeZone("GMT+8")
        }
    }

    /**
     * 注册Kotlin的Module
     */
    @Bean
    @Primary
    fun objectMapper(): ObjectMapper {
        val mapper: ObjectMapper = builder().build()
        mapper.registerKotlinModule()
        return mapper
    }

    /**
     * XML解析和生成
     */
    @Bean("xmlObjectMapper")
    fun xmlObjectMapper(): ObjectMapper {
        return Jackson2ObjectMapperBuilder.xml().build()
    }


    @Bean(CONVERTER_NAME)
    fun getConverter(objectMapper: ObjectMapper): MappingJackson2HttpMessageConverter {
        return MappingJackson2HttpMessageConverter().apply {
            this.objectMapper = objectMapper
        }
    }

    /**
     * 解析XML转换器，不知道能否成功
     */
    @Bean
    fun xmlConverter(@Qualifier("xmlObjectMapper") objectMapper: ObjectMapper): MappingJackson2XmlHttpMessageConverter {
        return MappingJackson2XmlHttpMessageConverter().apply {
            this.objectMapper = objectMapper
        }
    }


}