package cn.tonyandmoney.tuon.qywx.service.impl

import cn.tonyandmoney.tuon.core.error.OpException
import cn.tonyandmoney.tuon.core.utils.RestUtils
import cn.tonyandmoney.tuon.qywx.QywxProperties
import cn.tonyandmoney.tuon.qywx.bean.WxAccessToken
import cn.tonyandmoney.tuon.qywx.bean.WxUser
import cn.tonyandmoney.tuon.qywx.bean.WxUserInfo
import cn.tonyandmoney.tuon.qywx.service.IQywxService
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.data.redis.core.StringRedisTemplate
import reactor.core.publisher.Mono
import java.time.Duration

/**
 * 实现企业微信接口
 */
class QywxServiceImpl(private val stringRedisTemplate: StringRedisTemplate,
                      private val properties: QywxProperties) : IQywxService {

    companion object {
        const val ACCESS_TOKEN = "qywx:access_token"
        private val logger:Logger = LoggerFactory.getLogger(QywxServiceImpl::class.java.simpleName)
    }


    private fun getRedisAccessTokenKey(agentId: String): String {
        return String.format("%s:%s", ACCESS_TOKEN, agentId)
    }

    override fun getAccessToken(agentId: String): Mono<String> {
        return Mono.create<String> {
            val token = stringRedisTemplate.opsForValue().get(getRedisAccessTokenKey(agentId))
            if (token.isNullOrBlank()) {
                val params = mapOf("corpid" to properties.corpid, "corpsecret" to properties.corpSecret)
                val resp = RestUtils.get(properties.host, properties.accessTokenPath, params, WxAccessToken::class.java)
                if (resp != null && !resp.access_token.isNullOrBlank()) {
                    stringRedisTemplate.opsForValue().set(getRedisAccessTokenKey(agentId), resp.access_token, Duration.ofMinutes(100))
                    it.success(resp.access_token)
                } else {
                    it.error(RuntimeException("获取应用AccessToken 失败！"))
                }
            } else {
                it.success(token)
            }
        }
    }

    override fun getUserInfo(agentId: String, code: String): Mono<WxUserInfo> {
        return getAccessToken(agentId)
                .flatMap { token ->
                    Mono.create<WxUserInfo> {
                        val params = mapOf("access_token" to token, "code" to code)
                        val info = RestUtils.get(properties.host, properties.userInfoPath, params, WxUserInfo::class.java)
                        logger.info("getUserInfo: {} ,accessToken:{}",info,token)
                        if (info.isOk) {
                            it.success(info)
                        } else {
                            it.error(OpException(info.errcode, info.errmsg))
                        }
                    }
                }
    }

    override fun getUserById(agentId: String, userId: String): Mono<WxUser> {
        return getAccessToken(agentId)
                .flatMap { token ->
                    Mono.create<WxUser> {
                        val params = mapOf("access_token" to token, "userid" to userId)
                        val info = RestUtils.get(properties.host, properties.userGetPath, params, WxUser::class.java)
                        logger.info("getUserById agentId:{},userId:{},info:{},accessToken:{}",agentId,userId,info,token)
                        it.success(info)
                    }
                }
    }


}